@@ -19,22 +19,18 @@
 #
 from __future__ import absolute_import
 
-import glob
+import io
 import json
 import logging
 import os
 import six
-import yaml
 
 from vdsm import utils
-from vdsm.common.compat import pickle
+from vdsm.common.compat import Enum, pickle
 from vdsm.common.logutils import Suppressed
 from yajsonrpc.exception import JsonRpcInvalidParamsError
 
 
-VDSM_CACHE_DIR = '/var/cache/vdsm/schema'
-
-
 PRIMITIVE_TYPES = {'boolean': lambda value: isinstance(value, bool),
                    'float': lambda value: isinstance(value, float),
                    'int': lambda value: isinstance(value, int),
@@ -68,94 +64,33 @@ class MethodNotFound(Exception):
     pass
 
 
-def _load_yaml_file(file_name):
-    if hasattr(yaml, 'CLoader'):
-        loader = yaml.CLoader
-    else:
-        logging.warning("yaml CLoader not available, using yaml.Loader")
-        loader = yaml.Loader
-    yaml_file = yaml.load(file_name, Loader=loader)
-    return yaml_file
+class SchemaType(Enum):
+    VDSM_API = "vdsm-api"
+    VDSM_API_GLUSTER = "vdsm-api-gluster"
+    VDSM_EVENTS = "vdsm-events"
 
+    @staticmethod
+    def schema_dirs():
+        """
+        Schema dir can be one of: source dir, if we're running directly
+        within the source tree, or the directory in which they can be
+        found after installation.
+        """
+        local_path = os.path.dirname(__file__)
+        installed_path = os.path.join(local_path, '..', 'rpc')
+        return (local_path, installed_path)
 
-def _get_pickle_schema_path(yaml_schema_path):
-    basename = os.path.basename(yaml_schema_path)
-    file_name = os.path.splitext(basename)[0]
-    return os.path.join(VDSM_CACHE_DIR, file_name + '.pickle')
+    def path(self):
+        filename = self.value + ".pickle"
+        potential_paths = [os.path.join(dir_path, filename)
+                           for dir_path in SchemaType.schema_dirs()]
 
+        for path in potential_paths:
+            if os.path.exists(path):
+                return path
 
-def caches_up_to_date():
-    for path in find_all_schemas():
-        pickle_path = _get_pickle_schema_path(path)
-        if (not os.path.exists(pickle_path) or
-            round(os.stat(path).st_mtime, 2) !=
-                round(os.stat(pickle_path).st_mtime, 2)):
-            return False
-    return True
-
-
-def find_all_schemas():
-    schema_paths = []
-    localpath = os.path.dirname(__file__)
-    installedpath = os.path.join(localpath, '..', 'rpc')
-    for path in (localpath, installedpath):
-        for f in glob.glob(path + '/*.yml'):
-            schema_paths.append(os.path.join(path, f))
-
-    return schema_paths
-
-
-def find_schema(schema_name='vdsm-api'):
-    """
-    Find the API schema file whether we are running from within the source
-    dir or from an installed location
-    """
-
-    localpath = os.path.dirname(__file__)
-    installedpath = os.path.join(localpath, '..', 'rpc')
-    for directory in (localpath, installedpath):
-        path = os.path.join(directory, schema_name + '.yml')
-        # we use source tree and deployment directory
-        # so we need to check whether file exists
-        if os.path.exists(path):
-            return path
-
-    raise SchemaNotFound("Unable to find API schema file in %s or %s" %
-                         (localpath, installedpath))
-
-
-def create_cache():
-    if not os.path.exists(VDSM_CACHE_DIR):
-        os.makedirs(VDSM_CACHE_DIR)
-    for path in find_all_schemas():
-        with open(path) as f:
-            loaded_schema = _load_yaml_file(f)
-            pickle_schema_path = os.path.join(
-                VDSM_CACHE_DIR, _get_pickle_schema_path(path))
-            if (os.path.exists(pickle_schema_path) and
-                    round(os.stat(path).st_mtime, 2) ==
-                    round(os.stat(pickle_schema_path).st_mtime, 2)):
-                logging.info(
-                    'Pickled schema {} already exists and up to date'.format(
-                        os.path.basename(pickle_schema_path)))
-                continue
-
-            logging.info('Writing new schema {}'.format(
-                os.path.basename(pickle_schema_path)))
-            with open(pickle_schema_path, 'wb') as pickled_schema:
-                pickle.dump(loaded_schema,
-                            pickled_schema,
-                            protocol=pickle.HIGHEST_PROTOCOL)
-
-            mod_time = os.stat(path).st_mtime
-            os.utime(pickle_schema_path, (mod_time, mod_time))
-            os.chmod(pickle_schema_path, 0o444)
-
-
-def remove_cache():
-    for f in glob.glob(VDSM_CACHE_DIR + '/*.pickle'):
-        logging.info("Removing schema file {}".format(f))
-        os.remove(os.path.join(VDSM_CACHE_DIR, f))
+        raise SchemaNotFound("Unable to find API schema file, tried: %s" %
+                             ", ".join(potential_paths))
 
 
 class MethodRep(object):
@@ -190,10 +125,10 @@ class Schema(object):
 
     log = logging.getLogger("SchemaCache")
 
-    def __init__(self, paths, strict_mode):
+    def __init__(self, schema_types, strict_mode):
         """
-        Constructs schema object based on yaml files provided as
-        list of paths and a mode which determines request/response
+        Constructs schema object based on an iterable of schema type
+        enumerations and a mode which determines request/response
         validation behavior. Usually it is based on api_strict_mode
         property from config.py
         """
@@ -201,15 +136,9 @@ class Schema(object):
         self._methods = {}
         self._types = {}
         try:
-            for path in paths:
-                pickle_path = _get_pickle_schema_path(path)
-
-                if not os.path.exists(pickle_path):
-                    with open(path) as f:
-                        loaded_schema = _load_yaml_file(f)
-                else:
-                    with open(pickle_path) as f:
-                        loaded_schema = pickle.load(f)
+            for schema_type in schema_types:
+                with io.open(schema_type.path(), 'rb') as f:
+                    loaded_schema = pickle.load(f)
 
                 types = loaded_schema.pop('types')
                 self._types.update(types)
@@ -217,6 +146,17 @@ class Schema(object):
         except EnvironmentError:
             raise SchemaNotFound("Unable to find API schema file")
 
+    @staticmethod
+    def vdsm_api(strict_mode, *args, **kwargs):
+        schema_types = {SchemaType.VDSM_API}
+        if kwargs.pop('with_gluster', False):
+            schema_types.add(SchemaType.VDSM_API_GLUSTER)
+        return Schema(schema_types, strict_mode, *args, **kwargs)
+
+    @staticmethod
+    def vdsm_events(strict_mode, *args, **kwargs):
+        return Schema((SchemaType.VDSM_EVENTS,), strict_mode, *args, **kwargs)
+
     def get_args(self, rep):
         method = self.get_method(rep)
         return method.get('params', [])
