@@ -21,7 +21,7 @@ from __future__ import absolute_import
 from importlib import import_module
 from pkgutil import iter_modules
 
-from vdsm.utils import memoized
+from vdsm.common.cache import memoized
 
 
 class NoDriverError(Exception):
